{% extends "../../../layouts/post.html" %}

{% block article %}
# A Query about Game State
## Preface
I'm an indie game dev that has an interest in making tools that I think would improve my life as a game developer in the hopes that it can help others. I'm not an expert in any of the topics that this article covers. I have no formal education in game engine development or architecture, systems programming, or anything else. I went to university for Computer Science with a specialization in game design & development when I was in my early 20s and the only thing I learned is that there are better ways to spend tens of thousands of dollars.

Everything in this article is drawn directly from my experiences working in tech and game development.

## Theseus' Shield
In the context of game design and development, object oriented data modeling can feel intuitive. Players, NPCs, and Enemies can all derive from a common "Creature" class which handle things like health and damage. Weapons, armor, and consumables can derive from a common Item class which handle inventory management.

Thinking exclusively in terms of hierarchal, nested patterns can come at a cost: software brittleness and inflexible design. Let's use our base `Item` class as an example.

```cs
abstract class Item {
    public string Name { get; protected set; }
}

abstract class Weapon: Item {
    public float Damage { get; protected set; }
    public float Range { get; protected set; }
}

abstract class Armor: Item {
    public float Defense { get; protected set; }
}

class Sword: Weapon {}
class Bow: Weapon {}

class Helmet: Armor {}
class Shield: Armor {}
```

In the above scenario, we have [three layers of inheritance](https://wiki.c2.com/?MaxThreeLayersOfInheritance). We can imagine that `Item` handles behaviors like item management such as being added or removed from an inventory and highly generic item behavior. `Weapon` would then be responsible for an item which is capable of dealing damage at any range. We can expect that the specifics of its attack behavior would be implemented by a subclass. Similarly for `Armor`, this would handle generic damage interactions such as common mitigation or avoidance calculations while leaving specific implementation details to its subclasses.

Seasoned game devs, disciplined object-oriented programmers, and existing ECS developers can likely already see what comes next. A new requirement.

How do we add a Spiked Shield item -- one which derives the behaviors of `Weapon` and `Armor`? If we were using a language which supports multiple inheritance this could potentially be a nonissue: except that multiple inheritance is often [purposefully missing](https://en.wikipedia.org/wiki/Multiple_inheritance#The_diamond_problem) in many languages. We could decide that the item belongs more to one class than the other and just duplicate the missing behavior but these types of decisions often introduce unforeseen complexity: will the damage algorithm need to do a specific type check for `SpikedShield`? What about the equipment screen? And of course, what happens when we need to implement a damaging potion?

Perhaps the most appropriate solution this case would be to forego the inheritance pattern in favor of [composition](https://en.wikipedia.org/wiki/Composition_over_inheritance). In C#, this could be achieved with `interfaces` and default implementations.

```cs
interface ICollectable {
    void Take();
    void Remove();
}

interface IDamaging {
    void ApplyDamage(float value) {
        // ...
    }
}

interface IDamageable {
    void ReceiveDamage(float value) {
        // ...
    }
}

class SpikedShield: ICollectable, IDamaging, IDamageable {}
```

We can use this approach to refactor our existing items and systems to derive/override only the behaviors they use. The respective systems for these interfaces now only need to check for the existence of these interfaces in order to act on them.

Engines like Unity and Godot use variations of the [Entity-Component (EC)](https://gameprogrammingpatterns.com/component.html) pattern (similar to but distinct from Entity-Component-System (ECS)). These patterns favor composition over inheritance by allowing developers to isolate behaviors into discrete components that can be applied to entities. In the Spiked Shield example, a developer could make a "Damage Source" and "Damage Target" component and add both to the item. In essence, this is the same as the interface-based approach.

Unfortunately, these patterns suffer a much bigger issue that is much more difficult to solve. 

## In search of loot
Managing game state is difficult. As a game grows, so too does the amount of amount of objects active at any given time. Dozens, hundreds, or even thousands of entities each with their own behaviors, goals, and concerns that interact with each other in different ways. This explosion in complexity can be exceedingly hard to manage even in games that are small in scope. Games, even at their simplest, tend to be *highly* complex.

## Enter ECS
ECS solves many of the problems presented by object oriented programming patterns but it's still in its infancy. 

## Issues with ECS
Mental modeling is hard. Complex queries are hard. ECS is hard.

## A Solution?

## Going forward

## What I'm looking for


{% endblock article %}
